/****************************************************************************/
/*                  Copyright 1997, Trustees of Boston University.          */
/*                               All Rights Reserved.                       */
/*                                                                          */
/* Permission to use, copy, or modify this software and its documentation   */
/* for educational and research purposes only and without fee is hereby     */
/* granted, provided that this copyright notice appear on all copies and    */
/* supporting documentation.  For any other uses of this software, in       */
/* original or modified form, including but not limited to distribution in  */
/* whole or in part, specific prior permission must be obtained from Boston */
/* University.  These programs shall not be used, rewritten, or adapted as  */
/* the basis of a commercial software or hardware product without first     */
/* obtaining appropriate licenses from Boston University.  Boston University*/
/* and the author(s) make no representations about the suitability of this  */
/* software for any purpose.  It is provided "as is" without express or     */
/* implied warranty.                                                        */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Utility Name: Files:  Generates set of properly named/sized files        */
/* Authors:      Paul Barford                                               */
/* Rev:    v1.1  4/22/98                                                    */
/*                                                                          */
/*                                                                          */
/* Description:                                                             */
/*                                                                          */
/* This program is part of SURGE which enables generation of a set of       */
/* files with appropriate sizes and names.  This file is meant to be copied */
/* to the server and placed in the directory to which SURGE will make       */
/* requests.  This program will generate the set of file requested by SURGE.*/
/* The files use names 0.txt to {num files}.txt where each file name        */
/* corresponds to the ordering for files found from top to bottom in        */
/* mout.txt.  The sizes of the files are equal to those listed for each in  */
/* mout.txt as well.                                                        */
/*                                                                          */
/* Inputs:                                                                  */
/*                                                                          */
/* The parameters for the program are listed in the DEFINE area of the      */
/* program header.  The input file used in this program is mout.txt which   */
/* is generated by match.c  mout.txt has format <number of references>,     */
/* <file size>, <file type>                                                 */
/*                                                                          */
/* Compile this program on the server under test by typing the following    */
/* (assuming you have a gnu compiler on your system):                       */
/*                                                                          */
/*                     gcc -Wall -o files files.c                           */
/*                                                                          */
/* Run this program by typing the following:                                */
/*                                                                          */
/*                              files                                       */
/*                                                                          */
/* Outputs:                                                                 */
/*                                                                          */
/* The program outputs a set of files each with proper names and sizes as   */
/* required by the SURGE.  The files themselves simply contain the letter   */
/* "a" which is only used to take up the right amount of space.             */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#define MAXTRACE 10000      /*   MAX number of input files handled          */

/* Define the routines used in this program     */

void getinputs();
void initialize();

/* Define global variables used in this program */

long num_infiles;           /* total number of input values                 */
long size[MAXTRACE];        /* file sizes from yout.txt                     */

/****************************************************************************/
/* Main simply generates the required output files                          */
/****************************************************************************/

void main()
{

  int i, x, y;
  char name[25];
  FILE *fp1;

/****************************************************************************/
/* Initialize arrays.                                                       */
/****************************************************************************/

  initialize();

/****************************************************************************/
/* Read input file.                                                         */
/****************************************************************************/

  getinputs();

/****************************************************************************/
/* Now generate the files giving them the proper names and sizes.  Place    */
/* all in the same directory (for now!).                                    */
/****************************************************************************/

  printf("Generating files\n");
  for(i=0;i < num_infiles;i++) 
    {
      sprintf (name, "%d.txt", i);
      if((fp1 = fopen(name, "w")) == NULL)
	{
	  perror("Unable to open filename.txt\n");
	  exit(1);
	}
      x = 0;
      y = 0;
      while(x < size[i])
	{
	  if (y < 80)
	    {
	      fprintf(fp1,"a");
	      y++;
	      x++;
	    } else {
	      fprintf(fp1,"\n");
	      y = 0;
	      x++;
	    }
	}
      fclose(fp1);
    }
 
} /* End Main */

/****************************************************************************/
/* The getinputs routine reads the values from mout.txt                     */
/****************************************************************************/

void getinputs()
{

  long refno, insize, grpno;
  FILE *fp;

  /* Get input from file mout.txt */

  if((fp = fopen("mout.txt", "r")) == NULL)
    {
      perror("Unable to open mout.txt\n");
      exit(1);
    }

  while( fscanf(fp, "%ld %ld %ld", &refno, &insize, &grpno) != EOF)
    {
      if(num_infiles > MAXTRACE)
	{
	  printf("Buffers filled before end of scan. Increase MAXTRACE\n");
	  exit(1);
	  }
      size[num_infiles] = insize;
      num_infiles++;
    }

  /* Close the output file  */

  fclose(fp);

} /* End getinputs */


/****************************************************************************/
/* Initializes the data array                                               */
/****************************************************************************/

void initialize()
{

  long i;

/* Initialize the sizes array to 0 */

  for(i=0;i<MAXTRACE;i++)
    {
      size[i] = 0;
    }

} /* End initialize */

