/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of VideoCM. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/*
 *  fileIO.cc
 *  Nick Feamster (feamster@lcs.mit.edu)
 *  MIT Lab for Computer Science
 *  created June 2000
 */


#include <stdio.h>
#include "fileIO.h"
							  
int FileIO::openBinaryRead() {
  fp = fopen(filename, "rb");
  if (fp)
    return FILE_SUCCESS;
  else
    return FILE_ERROR;
}

int FileIO::openBinaryWrite() {
  fp = fopen(filename, "ba");
  if (fp)
    return FILE_SUCCESS;
  else
    return FILE_ERROR;
}

int FileIO::openBinaryReadWrite() {
  fp = fopen(filename, "ba+");
  if (fp)
    return FILE_SUCCESS;
  else
    return FILE_ERROR;
}
							  
int FileIO::read(char *buf, int sz) {
  int ret;
  
  if (!fp) {
    // error
    fprintf(stderr, "Error: file stream ptr not initialized!\n");
    return FILE_ERROR;
  }
  if (!(ret = fread(buf, sizeof(char), sz, fp))) {
    // error
    fprintf(stderr, "Error: Error reading file into buffer!\n");
    return FILE_ERROR;
  }
  return ret;
}

int FileIO::write(char *buf, int sz) {
  int ret;
  
  if (!fp) {
    // error
    fprintf(stderr, "Error: file stream ptr not initialized!\n");
    return FILE_ERROR;
  }
  if (!fwrite(buf, sizeof(char), sz, fp)) {
    // error
    fprintf(stderr, "Error: Error writing file from buffer!\n");
    return FILE_ERROR;
  }
  return ret;
}

int FileIO::isEOF() {
  if (!fp) {
    // error
    perror("Error: file stream ptr not initialized!");
    return 1;
  } else {
    return feof(fp);
  }
}


unsigned char FileIO::peekCurrentByte() {
  unsigned char ret = fgetc(fp);
  ungetc((int)ret, fp);
  return ret;
}

int FileIO::rewind() {
  (void)fseek(fp, 0L, SEEK_SET);
  return FILE_SUCCESS;
}


int FileIO::advance(long bytes) {
  if (!fseek(fp,bytes, SEEK_CUR))
    return FILE_SUCCESS;
  else
    return FILE_ERROR;
}

