/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of VideoCM. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/*
 * Nick Feamster (feamster@lcs.mit.edu)
 * MIT LCS
 * created January 2001
 */

#include "resync.h"
#include "fileIO.h"




void
flush_until_resync(Bitstream *stream) {
  BitstreamFlushBits(stream, 17);
  do {
    BitstreamFlushBits(stream, 1);
  } while ((BitstreamShowBitsByteAlign(stream, 17) != RESYNC_MARKER));
 
}



int
read_until_vop(Bitstream *stream, char *vop_buf, int maxsz) {

  char tmpvar;
  int i=0, tmpvar2;
  char *s = vop_buf;

  do {
    tmpvar = BitstreamReadBits(stream, sizeof(char)*8, NULL, NULL, 0);
    memcpy(vop_buf+i, (char*)(&tmpvar), sizeof(char));
    i+=sizeof(char);
  } while ((BitstreamShowBitsByteAlign(stream, 32) != VOP_START_CODE) &&
	   i<maxsz && !feof(stream->fptr));
  

  if (!(i<maxsz))
    fprintf(stderr, "WARNING: not enough memory allocated!\n");

  if (feof(stream->fptr)) {
    fprintf(stderr, "EOF: %d\n", i);
  }
  return i;
}


void vop_sizes(char *filename, char *outfile) {
  Bitstream *stream = BitstreamOpen(filename);
  FILE *fp = stream->fptr;
  int i;

  FILE *fpo = fopen(outfile, "wb");

  char *vop_buf = (char *)malloc(100*PACKET_SIZE*sizeof(char));

  while (!feof(fp)) {
    i = read_until_vop(stream, vop_buf, 100*PACKET_SIZE);
    fprintf (stdout, "FOUND VOP %d\n", i);
    fwrite(vop_buf, i, sizeof(char), fpo);
    fflush(fpo);
  }
  free(vop_buf);
  fclose(fpo);
}


int
read_until_resync(Bitstream *stream, char *vop_buf, int maxsz) {

  char tmpvar;
  int i=0, tmpvar2;
  char *s = vop_buf;

  
  if ((BitstreamShowBitsByteAlign(stream, 17) != RESYNC_MARKER))
    fprintf(stderr, "WARNING: First marker should be resync!\n");

  do {
    tmpvar = BitstreamReadBits(stream, sizeof(char)*8, NULL, NULL, 0);
    memcpy(vop_buf+i, (char*)(&tmpvar), sizeof(char));
    i+=sizeof(char);
  } while ((BitstreamShowBitsByteAlign(stream, 17) != RESYNC_MARKER) &&
	   i<maxsz && !feof(stream->fptr));
  

  if (!(i<maxsz))
    fprintf(stderr, "WARNING: not enough memory allocated!\n");

  if (feof(stream->fptr)) {
    fprintf(stderr, "EOF: %d\n", i);
  }
  return i;
}


int
read_until_resync_or_vop(Bitstream *stream, char *vop_buf, int maxsz) {

  char tmpvar;
  int i=0, tmpvar2;
  char *s = vop_buf;

  do {
    tmpvar = BitstreamReadBits(stream, sizeof(char)*8, NULL, NULL, 0);
    memcpy(vop_buf+i, (char*)(&tmpvar), sizeof(char));
    i+=sizeof(char);
  } while ((BitstreamShowBitsByteAlign(stream, 17) != RESYNC_MARKER) &&
	   (BitstreamShowBitsByteAlign(stream, 32) != VOP_START_CODE) &&
	   i<maxsz && !feof(stream->fptr));
  

  if (!(i<maxsz))
    fprintf(stderr, "WARNING: not enough memory allocated!\n");

  if (feof(stream->fptr)) {
    fprintf(stderr, "EOF: %d\n", i);
  }
  return i;
}




int
count_resync_bytes(char *filename) {

  int i, bytes=0, packets=0;
  Bitstream *stream = BitstreamOpen(filename);

  
  while (!feof(stream->fptr)) {
    i=0;
    if ((BitstreamShowBitsByteAlign(stream, 17) != RESYNC_MARKER))
      fprintf(stderr, "WARNING: First marker should be resync!\n");
    
    do {
      BitstreamReadBits(stream, sizeof(char)*8, NULL, NULL, 0);
      //      memcpy(vop_buf+i, (char*)(&tmpvar), sizeof(char));
      i+=sizeof(char);
      
    } while ((BitstreamShowBitsByteAlign(stream, 17) != RESYNC_MARKER));
//      fprintf(stderr, "%d\n", i);

    if (i<2000) {
      bytes+=i;
      packets++;
    }
  }
  fprintf(stderr, "%f\n", (double)bytes/(double)packets);
  return i;
}


void
copy_file(FILE *fi, FILE *fo) {
   char val;

   do {
     fread(&val, sizeof(char), 1, fi);
     fwrite(&val, sizeof(char), 1, fo);
   } while (!feof(fi));
}


void drop_read_until_vop(Bitstream *stream, FILE *fpo,
			 double drop_prob, int type) {

  FILE *fp = stream->fptr;
  srand(time(NULL));
   
  char tmpvar;
  int last_type=-1, ret;

  char *vop_buf = (char *)malloc(100*PACKET_SIZE*sizeof(char));

  while (!feof(fp)) {
  
    ret = read_until_resync_or_vop(stream, vop_buf, 100*PACKET_SIZE);
    
    if ((double)(rand())/(RAND_MAX) < drop_prob ||
	(last_type==type)) {
      fwrite(vop_buf, sizeof(char), ret, fpo);
    } else {
      fprintf(stderr, "DROP!\n");
    }

  
    if (BitstreamShowBitsByteAlign(stream,32) == VOP_START_CODE) {
      for (int i=0; i<4; i++) {
	tmpvar = BitstreamReadBits(stream, 8, NULL, NULL, 0);
	//        fread(&tmpvar, sizeof(char), 4, fp);
	fwrite(&tmpvar, sizeof(char), 1, fpo);
	fflush(fpo);
      }

      tmpvar = BitstreamShowBitsByteAlign(stream, 2);
      
      switch (tmpvar) {
      case I_VOP:
	fprintf(stderr, "reached I frame\n");
	break;
      case P_VOP:
	fprintf(stderr, "reached P frame\n");
	break;
      case B_VOP:
	fprintf(stderr, "reached B frame\n");
	break;
      default:
	fprintf(stderr, "UNKNOWN FRAME TYPE!\n");
      }
      
      last_type = (int)tmpvar;
    }
  }

  copy_file(fp, fpo); /* just to make sure, copy out until eof */
}


  

void random_packet_drop_by_frame(char *filename, char *outfile,
				 double drop_prob, int frame_type) {
   Bitstream *stream = BitstreamOpen(filename);
   int type, count=0;
   FILE *fpo = fopen(outfile, "wb");

   drop_read_until_vop(stream, fpo, drop_prob, frame_type);


   return;

}


void random_packet_drop(char *filename, char *outfile, double drop_prob) {
  int total=0;
   FILE *fp, *fpo;
//     fp = fopen(filename, "rb");

   Bitstream *stream = BitstreamOpen(filename);
   fp = stream->fptr;
   fpo = fopen(outfile, "wb");
   srand(time(NULL));

   char *filebuf = (char *)malloc(PACKET_SIZE*sizeof(char));

   /* always copy through first sync marker. */
   for (int i=0; i<1; i++) {
     int ret = read_until_resync(stream, filebuf, PACKET_SIZE);
//       int ret = copy_to_sync(fp, filebuf, PACKET_SIZE);
     fwrite(filebuf, sizeof(char), ret, fpo);
   }

   while (!feof(fp)) {
     //       int ret = read_until_resync(stream, filebuf, PACKET_SIZE);
     //       int ret = copy_to_sync(fp, filebuf, PACKET_SIZE);

     if ((double)(rand())/(RAND_MAX) < drop_prob) {
       int ret = read_until_resync(stream, filebuf, PACKET_SIZE);

       if (ret < 0)
	 break;
       fwrite(filebuf, sizeof(char), ret, fpo);
       //        total+=ret;
       //        fprintf(stdout, "output: %d %d\n", ret, total);
     } else {
       flush_until_resync(stream);
       fprintf(stderr, "DROP!\n");
     }
   }
   copy_file(fp, fpo); /* just to make sure, copy out until eof */
   return;
}


/********************* end of useful functions ***********************/

 void main (int argc, char **argv) {
   char *filename = argv[1];
   FILE *fp = fopen(filename, "rb");

   //print_hex(fp);
   //read_to_vop_start(fp);
   //read_to_sync(fp);
   //read_all_syncs(fp);

   int power = atoi(argv[3]);
   double prob = 1 - pow(2,-power);

     random_packet_drop(argv[1],argv[2],prob);
//     random_packet_drop_by_frame(argv[1],argv[2], prob, I_VOP);
//   vop_sizes(argv[1], argv[2]);

//     count_resync_bytes(filename);

}



/**************** OLD stuff below ****************************/
  




 void read_to_vop_start(FILE *fp) {
   unsigned char ret;
   int count=0;

   while (!feof(fp)) {
     int zeros = 0;

     while (zeros<4 && !feof(fp)) {
       char val = (char)fgetc(fp);
       if ((char)val == EOF)
	 break;

       if (!val)
	 zeros+=2;
       else
	 break;
     }

     if (zeros != 4) {
       if (zeros)
	 //fprintf (stdout, "zeros: %d\n", zeros);
       continue;
     } else {
       ret = fgetc(fp);
       if (ret == 1) {
	 fprintf(stdout, "FOUND VOP %d!\n", count++);
       }
     }
     zeros = 0;
   }
 }


 /* read file to 0000 0000 0000 0000 1 => resync marker */
 void read_to_sync(FILE *fp) {

   unsigned char val;
   unsigned char ret;
   int count=0;

   while (!feof(fp)) {
     int zeros = 0;

     while (zeros<4 && !feof(fp)) {
       fread(&val, sizeof(char), 1, fp);
       if ((char)val == EOF)
	 break;

       if (!((val & 0xF0)>>4))
	 zeros++;
       if (!(val & 0xF))
	 zeros++;
       else
	 break;
     }

     if (zeros != 4) {
       continue;
     } else {
       fread(&ret, sizeof(char), 1, fp);
       if (ret >> 7) {
	 fprintf(stderr, "READ TO RESYNC %d!\n", count++);
	 return;
       }
     }
     zeros = 0;
   }
 }


 /* read file to 0000 0000 0000 0000 1 => resync marker */
 void read_all_syncs(FILE *fp) {

   unsigned char val;
   unsigned char ret;
   int count=0;

   while (!feof(fp)) {
     int zeros = 0;

     while (zeros<4 && !feof(fp)) {
       fread(&val, sizeof(char), 1, fp);
       if ((char)val == EOF)
	 break;

       if (!((val & 0xF0)>>4))
	 zeros++;
       if (!(val & 0xF))
	 zeros++;
       else
	 break;
     }

     if (zeros != 4) {
       //fprintf (stderr, "zeros: %d\n", zeros);
       continue;
     } else {
       fread(&ret, sizeof(char), 1, fp);
       if (ret >> 7) {
	 fprintf(stderr, "READ TO RESYNC %d!\n", count++);
       }
     }
     zeros = 0;
   }
 }




int copy_to_sync(FILE *fp, char *buf, int sz) {

   unsigned char val;
   unsigned char ret;
   int count=0, i=0, intval;

   while (!feof(fp)) {
     int zeros = 0;

     if (i > sz) {
       fprintf(stderr, "error: not enough memory allocated!\n");
       return i;
     }

 //      fread(&intval, sizeof(int), 1, fp);
 //      memcpy((char*)(buf+i), &val, sizeof(int));
 //      i+=sizeof(int);

     while (zeros<4 && !feof(fp)) {
       fread(&val, sizeof(char), 1, fp);
       memcpy((char*)(buf+i), &val, 1);
       i++;

       if ((char)val == EOF)
	 break;

       if (!((val & 0xF0)>>4))
	 zeros++;
       if (!(val & 0xF))
	 zeros++;
       else
	 break;
     }

     if (zeros != 4) {
       continue;
     } else {
       fread(&ret, sizeof(char), 1, fp);
       memcpy((char*)(buf+i), &ret, 1);
       i++; 
       if (ret >> 7) {
	 return i;
       }
     }
     zeros = 0;
   }
 }




 void print_hex(FILE *fp) {
   char val;
   int count, total=0;
   while (!feof(fp)) {
     int count = fread(&val, sizeof(char), 1, fp);
     fprintf (stdout, "%.2x\t", val);
     total += count;
   }
   printf("\ntotal bytes: %d\n", total);
 }

