/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of bgptools. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     Nick Feamster (feamster@lcs.mit.edu)
 *     MIT Lab for Computer Science
 *     November 2001
 */

#include "config.h"

#ifdef __USE_GTK__

#include "gmain.h"
#include <stdio.h>
#include <sys/file.h>
#include <netinet/in.h>

GtkWidget *window;
GtkWidget *label0, *label1, *label1b, *label2, *label2b,
  *label3, *label3b, *label4, *label4b, *label5, *label5b;

GtkWidget *hsep0;

GtkWidget *pbox;
GtkWidget *box1, *box2, *box3, *box4, *box5;                           



void
usage(char **argv) {
  printf("%s <statfile>\n", argv[0]);
  exit(0);
}

gint
check_stats(void *data) {
  FILE *fp;
  char *filename = (char*)data;
  char tmp[80];

  u_int32_t adu_received, adu_lost, pkts_lost, pkts_received, retx;

//    fprintf(stderr, "checking stats from %s...\n", filename);
 
  fp = fopen(filename, "rb");

  if (!fp) {
    fprintf(stderr, "error opening stat file\n");
    return 0;
  }

  flock(fileno(fp), LOCK_EX);

  
  fread(&adu_received, sizeof(u_int32_t), 1, fp);
  adu_received = ntohl(adu_received);
  
  fread(&adu_lost, sizeof(u_int32_t), 1, fp);
  adu_lost = ntohl(adu_lost);

  fread(&pkts_received, sizeof(u_int32_t), 1, fp);
  pkts_received = ntohl(pkts_received);

  fread(&pkts_lost, sizeof(u_int32_t), 1, fp);
  pkts_lost = ntohl(pkts_lost);
  
  fread(&retx, sizeof(u_int32_t), 1, fp);
  retx = ntohl(retx);


  sprintf(tmp, "%lu", adu_received);
  gtk_label_set_text(GTK_LABEL(label4b), tmp);

  sprintf(tmp, "%lu", adu_lost);
  gtk_label_set_text(GTK_LABEL(label5b), tmp);
  
  sprintf(tmp, "%lu", pkts_received);
  gtk_label_set_text(GTK_LABEL(label1b), tmp);

  sprintf(tmp, "%lu", pkts_lost);
  gtk_label_set_text(GTK_LABEL(label2b), tmp);

  sprintf(tmp, "%lu", retx);
  gtk_label_set_text(GTK_LABEL(label3b), tmp);


  flock(fileno(fp), LOCK_UN);
  fclose(fp);
  return 1;
}


int main( int   argc,
	  char *argv[] )
{

  if (argc < 2) {
    usage(argv);
  }


  gtk_init (&argc, &argv);
                             
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
//    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
//  		      GTK_SIGNAL_FUNC (delete_event), NULL);
//    gtk_signal_connect (GTK_OBJECT (window), "destroy",
//    		      GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);

  pbox = gtk_vbox_new (FALSE, 0);

  /*************************************************************/


  label0 = gtk_label_new("SR-RTP Statistics");
  gtk_label_set_justify(GTK_LABEL(label0), GTK_JUSTIFY_CENTER);
//    gtk_misc_set_alignment (GTK_MISC (label0), 0, 0);
  gtk_box_pack_start (GTK_BOX (pbox), label0, FALSE, FALSE, 0);
  gtk_widget_show(label0);

  hsep0 = gtk_hseparator_new();
  gtk_box_pack_start (GTK_BOX (pbox), hsep0, FALSE, FALSE, 0);
  gtk_widget_show(hsep0);
  
  /*************************************************************/

  box4 = gtk_hbox_new (FALSE, 0);

  label4 = gtk_label_new("Frames Received:");
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0);
  gtk_box_pack_start (GTK_BOX (box4), label4, FALSE, FALSE, 10);
  gtk_widget_show(label4);

  label4b = gtk_label_new("0");
  gtk_misc_set_alignment (GTK_MISC (label4b), 1, 0);
  gtk_box_pack_end (GTK_BOX (box4), label4b, FALSE, FALSE, 10);
  gtk_widget_show(label4b);

  gtk_box_pack_start (GTK_BOX (pbox), box4, FALSE, FALSE, 0);
  gtk_widget_show(box4);

  /*************************************************************/

  box5 = gtk_hbox_new (FALSE, 0);

  label5 = gtk_label_new("Frames Lost:");
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0);
  gtk_box_pack_start (GTK_BOX (box5), label5, FALSE, FALSE, 10);
  gtk_widget_show(label5);

  label5b = gtk_label_new("0");
  gtk_misc_set_alignment (GTK_MISC (label5b), 1, 0);
  gtk_box_pack_end (GTK_BOX (box5), label5b, FALSE, FALSE, 10);
  gtk_widget_show(label5b);

  gtk_box_pack_start (GTK_BOX (pbox), box5, FALSE, FALSE, 0);
  gtk_widget_show(box5);
  
  /*************************************************************/

  box1 = gtk_hbox_new (FALSE, 0);

  label1 = gtk_label_new("Packets Received:");
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0);
  gtk_box_pack_start (GTK_BOX (box1), label1, FALSE, FALSE, 10);
  gtk_widget_show(label1);

  label1b = gtk_label_new("0");
  gtk_misc_set_alignment (GTK_MISC (label1b), 1, 0);
  gtk_box_pack_end (GTK_BOX (box1), label1b, FALSE, FALSE, 10);
  gtk_widget_show(label1b);

  gtk_box_pack_start (GTK_BOX (pbox), box1, FALSE, FALSE, 0);
  gtk_widget_show(box1);

  /*************************************************************/

  box2 = gtk_hbox_new (FALSE, 0);

  label2 = gtk_label_new("Packets Lost:");
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0);
  gtk_box_pack_start (GTK_BOX (box2), label2, FALSE, FALSE, 10);
  gtk_widget_show(label2);

  label2b = gtk_label_new("0");
  gtk_misc_set_alignment (GTK_MISC (label2b), 1, 0);
  gtk_box_pack_end (GTK_BOX (box2), label2b, FALSE, FALSE, 10);
  gtk_widget_show(label2b);

  gtk_box_pack_start (GTK_BOX (pbox), box2, FALSE, FALSE, 0);
  gtk_widget_show(box2);

  /*************************************************************/

  box3 = gtk_hbox_new (FALSE, 0);

  label3 = gtk_label_new("Retransmission Requests:");
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0);
  gtk_box_pack_start (GTK_BOX (box3), label3, FALSE, FALSE, 10);
  gtk_widget_show(label3);

  label3b = gtk_label_new("0");
  gtk_misc_set_alignment (GTK_MISC (label3b), 1, 0);
  gtk_box_pack_end (GTK_BOX (box3), label3b, FALSE, FALSE, 10);
  gtk_widget_show(label3b);

  gtk_box_pack_start (GTK_BOX (pbox), box3, FALSE, FALSE, 0);
  gtk_widget_show(box3);

  /*************************************************************/



  gtk_container_add(GTK_CONTAINER(window), pbox);
  gtk_widget_show(pbox);

  gtk_widget_show  (window);


  gtk_timeout_add(1000, check_stats, (void*)argv[1]);
  gtk_main ();
                             
  return(0);
}

#else

int
main(int argc, char **argv) {
  printf("Not compiled with GTK support!\n");
  exit(0);


#endif
