/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of VideoCM. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


#ifndef mash_pktbuf_h
#define mash_pktbuf_h

#ifdef WIN32
#include <winsock.h>
#else
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#endif
#include "mash_config.h"

#include "inet.h"


#include <stdio.h>

class RTP_pktbuf;


/*
 * The base object for performing the outbound path of
 * the application level protocol.
 */

class RTP_BufferPool_ {
    public:
	RTP_BufferPool_();
	void release(RTP_pktbuf*);

	/*
	 * Buffer allocation hooks.
	 */
	virtual RTP_pktbuf* alloc(unsigned char type=0);
	RTP_pktbuf* freebufs() { return freebufs_;}
    private:
	RTP_pktbuf* freebufs_;
	int nbufs_;
	int nused_;
};

/*XXX*/
#define MAXHDR 128
#define PKTBUF_PAD 256
#define PKTBUF_SIZE (MAXHDR + 20*1024 + PKTBUF_PAD)


//class RTP_pktbuf : public RTP_Buffer {   /* Removed by NGF 7/06/00 */

class RTP_pktbuf {
 public:
  RTP_pktbuf* next;
  int offset;     /*
		   * Byte offset of first byte in fragment. Used during
		   * reassembly. */

  int len;
  int layer;
  int ref;		   
  u_int32_t addr; /* address, */
  u_int16_t port; /* port, and */


  u_int8_t* dp;
  u_int8_t data[PKTBUF_SIZE];    /* will be case to struct rrtphdr */
  RTP_BufferPool_* manager;
  
  inline void release() {
    ref--;
    if (ref<=0)
      manager->release(this);
  }
  inline void attach() {
    ref++;
  }

  void update_rrtp_timestamp();

  /* Copy a pktbuf, but prepend a fresh cmn_hdr. */
  RTP_pktbuf* copy(u_char type);
};
#endif




