/*
 * Copyright (c) 2001 Massachusetts Institute of Technology.
 *
 * This file is part of VideoCM. 
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `COPYING' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file AUTHORS which should have accompanied this software 
 * for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the AUTHORS file.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


/*
 *     rtsp-player.cc
 *     Nick Feamster
 *     MIT Lab for Computer Science
 *     created July 2000
 */

#include "rtsp.h"
#include <tcl.h>
#include <tk.h>

RTSP_Player *rtsp_play;

void srrtp_app_notify(rtp_channel_t ch, int seqno, int len, int layer) {
#ifdef SRRTP_DEBUG_ENABLED
  printf("called rtp_app_notify: seq %d, length %d\n", seqno, len);
#endif
  rtsp_play->udpread_callback(seqno, len);
}

void srrtp_app_send(rtp_channel_t ch) {
#ifdef SRRTP_DEBUG_ENABLED
  printf("called rtp_app_send\n");
#endif
}

#ifdef _USE_CM_
void cmapp_send(int cmid) {
#ifdef SRRTP_DEBUG_ENABLED
  fprintf(stderr, "call to cmapp_send...\n");
#endif
}

void cmapp_update(int cmid, cmquery cminfo) {
#ifdef SRRTP_DEBUG_ENABLED
  fprintf(stderr, "call to cmapp_update...\n");
#endif
}

void app_layer_callback(double rate) {
}
#endif

void do_eventloop() {
  while(1) {
#ifdef TCL_GUI
    Tcl_DoOneEvent(TCL_WINDOW_EVENTS | TCL_DONT_WAIT);
#endif
    Tcl_DoOneEvent(0);
  }
}

void stdin_callback() {
  char buffer[256];
  fgets(buffer, sizeof(buffer), stdin);
  rtsp_play->handle_command(buffer);
}



int main(int argc, char **argv) {
  Tcl_CreateFileHandler(1, TCL_READABLE, (Tcl_FileProc*)stdin_callback, NULL);
  rtsp_play =  new RTSP_Player();

  puts( "enter command ('?' or 'h' for help): " );
  do_eventloop();

  delete rtsp_play;
}
