/******************************************************************
 *
 * Nick Feamster
 * MIT LCS
 * August 2000
 * psnr.c
 *
 ******************************************************************/

#include <stdio.h>
#include <math.h>
#include "psnr.h"


float psnr(unsigned char *img1, unsigned char *img2, int nrows, int ncols)
{
  int i, j;
  float msd, psnr;

  msd = 0.0;

  for(i=0;i<nrows;i++)
    for(j=0;j<ncols;j++)
      msd += (float) ((*(img1+i*ncols+j) - *(img2+i*ncols+j))*
		       (*(img1+i*ncols+j) - *(img2+i*ncols+j)));

  psnr = PEAK*PEAK*(nrows*ncols)/msd;
  psnr = 10.0*log10((float) psnr);

  return(psnr);
}


float psnr_RGB(unsigned char *img1, unsigned char *img2, int nrows, int ncols)
{
  int i, j;
  return(psnr(img1, img2, nrows, ncols*3));
}



void psnr_YUV420(unsigned char *img1, unsigned char *img2,
		 int nrows, int ncols,
		 float *psnr_y, float *psnr_u, float *psnr_v)
{
  int i, j, offset=0;
  float msd;
  int nrows_uv = nrows/2;
  int ncols_uv = ncols/2;

  
  /** Y Component **/
  *psnr_y = psnr(img1, img2, nrows, ncols);

  /** U Component **/
  offset = nrows*ncols;
  *psnr_u = psnr(img1+offset, img2+offset, nrows_uv, ncols_uv);


  /** V Component **/
  offset = nrows*ncols + (nrows_uv)*(ncols_uv);
  *psnr_v = psnr(img1+offset, img2+offset, nrows_uv, ncols_uv);

}




void psnr_file(char *filename1, char *filename2, int nrows, int ncols,
	       int t_offset, int b_offset) {

  FILE *fp1, *fp2;
  char *img1, *img2;
  char *p1, *p2;
  int size, type, cc1, cc2;
  int image_counter=0;

  float psnr_y, psnr_u, psnr_v;
  float psnr;

  p1 = (char*)strrchr(filename1, '.');
  p2 = (char*)strrchr(filename2, '.');
  if ((strcasecmp(p1, ".RGB") && strcasecmp(p1, ".YUV"))) {
    fprintf(stderr, "ERROR: files must be of RGB or YUV formats.\n");
    return;
  }
  if ((strcasecmp(p2, ".RGB") && strcasecmp(p2, ".YUV"))) {
    fprintf(stderr, "ERROR: files must be of RGB or YUV formats.\n");
    return;
  }
  if (strcasecmp(p1,p2)) {
    fprintf(stderr, "ERROR: files must have matching formats.\n");
    return;
  }
  if ((fp1 = fopen (filename1, "rb")) == NULL) {
    fprintf(stderr, "ERROR: couldn't open file: %s\n", filename1);
    return;
  }
  if ((fp2 = fopen (filename2, "rb")) == NULL) {
    fprintf(stderr, "ERROR: couldn't open file: %s\n", filename2);
    return;
  }
  

  if (!strcasecmp(p1, ".RGB"))
    type = IMG_RGB;
  else if (!strcasecmp(p1, ".YUV"))
    type = IMG_YUV;
  
  switch (type) {
  case IMG_YUV:
    size = nrows*ncols*3/2;
    break;
  case IMG_RGB:
    size = nrows*ncols*3;
    break;
  }

  img1 = (char*)malloc(size);
  img2 = (char*)malloc(size);
 
  
  while (!(feof(fp1) || feof(fp2))) {

    fseek(fp1, t_offset, SEEK_CUR);

    cc1 = fread(img1, sizeof(char), size, fp1);
    cc2 = fread(img2, sizeof(char), size, fp2);

    if (!cc1 || !cc2)
      break;

    switch (type) {
    case IMG_YUV:
      psnr_YUV420(img1, img2, nrows, ncols, &psnr_y, &psnr_u, &psnr_v);
      printf("%d\t%f\t%f\t%f\n", image_counter++, psnr_y, psnr_u, psnr_v);
      break;
    case IMG_RGB:
      psnr = psnr_RGB(img1, img2, nrows, ncols);
      printf("%d\t%f\n", image_counter++, psnr);
      break;
    }

    fseek(fp1, b_offset, SEEK_CUR);
    
  }

  free(img1);
  free(img2);
  fclose(fp1);
  fclose(fp2);

}



